# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
import http

from klatchat_utils.http_utils import respond


class KlatchatException(Exception):
    msg = "Klatchat Exception Occurred"

    def __init__(self, message=None):
        message = message or self.msg
        super().__init__(message)


class KlatAPIAuthorizationError(KlatchatException):
    msg = "Failed to authenticate in klat with provided credentials"


class MalformedConfigurationException(KlatchatException):
    msg = "Invalid configuration provided"


class KlatAPIException(Exception):

    HTTP_CODE = http.HTTPStatus.INTERNAL_SERVER_ERROR
    MESSAGE = "Internal Server Error"

    def __init__(self, message: str = None):
        self.MESSAGE = message or self.MESSAGE
        super().__init__(message)

    def to_http_response(self):
        return respond(msg=self.MESSAGE, status_code=self.HTTP_CODE.value)


class UserUnauthorizedException(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.UNAUTHORIZED
    MESSAGE = "Requested user is not authorized to perform this action"


class InvalidSessionTokenException(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.UNAUTHORIZED
    MESSAGE = "Session token is invalid or expired"


class ItemNotFoundException(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.NOT_FOUND
    MESSAGE = "Requested item not found"


class DuplicatedItemException(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.CONFLICT
    MESSAGE = "Requested item already exists"


class MissingAttributeException(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.UNPROCESSABLE_ENTITY
    MESSAGE = "Missing required attributes"


class InvalidInputData(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.BAD_REQUEST
    MESSAGE = "Wrong input data provided"


class PermissionDenied(KlatAPIException):
    HTTP_CODE = http.HTTPStatus.FORBIDDEN
    MESSAGE = "User has no permission to access this resource"

